import turtle
import imageio
import os
import sys
import glob
import shutil
import re

class Gif(object):
    def __init__(self, func, fps_for_eps, fps_for_gif):
        self.__func = func
        self.__running = True
        self.__fps_for_eps = fps_for_eps
        self.__fps_for_gif = fps_for_gif
        self.__cnt = 1
        save_name = './' + sys.argv[0][:-3] + '.gif'
        if not os.path.exists('tmp'):
            os.mkdir('tmp')

    # sort *.eps files by numerical order.
    def __numericalSort(self, value):
        parts = re.compile(r'(\d+)').split(value)
        parts[1::2] = map(int, parts[1::2])
        return parts

    def __draw(self):
        turtle.hideturtle()
        self.__func()
        turtle.ontimer(self.__stop, 500)
        turtle.down()

    def __stop(self):
        self.__running = False

    # save to temporary .eps files
    def __save(self):
        print('  Capturing picture %04d.' % self.__cnt)
        turtle.getcanvas().postscript(file = "./tmp/{0:04d}.eps".format(self.__cnt))
        self.__cnt += 1
        if self.__running:
            turtle.ontimer(self.__save, int(1000 / self.__fps_for_eps))
        else:
            turtle.ontimer(self.__convert, 500)

    def __convert(self):
        turtle.bye()
        print('Finished drawing and capturing.')
        print('Begin to convert the captured eps files to gif...')
        # read .eps files and change them to gif.
        images = []
        for file in sorted(glob.glob('./tmp/*.eps'), key=self.__numericalSort):
            images.append(imageio.imread(file))
        kwargs_write = {'fps': self.__fps_for_gif, 'quantizer': 'nq'}
        save_name = './' + sys.argv[0][:-3] + '.gif'
        imageio.mimsave(save_name, images, 'GIF-FI', **kwargs_write)
        shutil.rmtree('./tmp')
        print('Finished converting, have fun!')

    def run(self):
        print('Begin to draw and Capturing pictures...')
        self.__save()
        turtle.ontimer(self.__draw, 500)
        turtle.done()

def convert2gif(func, fps_for_eps=10, fps_for_gif=10):
    gif = Gif(func, fps_for_eps, fps_for_gif)
    gif.run()



import turtle

class Rabit(object):
    def __init__(self, grids, size=600):
        self.__grid_size = size / grids
        self.__left = -size / 2 - 5
        self.__top = size / 2 + 5
        self.__speed = 5
        self.__mat = [[{"text":"", "line_color":"black", "text_color":"black", "font":("Times", 24)}\
                       for _ in range(grids)] for _ in range(grids)]

        self.t = turtle.Turtle()
        self.t.pensize(2)
        self.t.hideturtle()
        self.t.speed(self.__speed)

    def speed(self, speed):
        self.__speed = speed
        self.t.speed(speed)

    def hold(self):
        turtle.done()

    def pensize(self, pensize):
       self.t.pensize(pensize) 

    def delay(self, time):
        self.begin_hide()
        turtle.delay(time)
        self.end_hide()
        
    def begin_hide(self):
        turtle.tracer(0)

    def end_hide(self):
        turtle.update()
        turtle.tracer(1)
        self.t.speed(self.__speed)

    def draw_rect(self, row, col, rows=1, cols=1, color="black"):
        self.t.pencolor(color)
        self.t.pu()
        self.t.setheading(0)
        self.t.goto(self.__left + col * self.__grid_size, self.__top - row * self.__grid_size) 
        self.t.pd()
        for i in range(2):
            self.t.fd(self.__grid_size * cols)
            self.t.rt(90)
            self.t.fd(self.__grid_size * rows)
            self.t.rt(90)

        for i in range(row, row + rows):
            for j in range(col, col + cols):
                    self.__mat[i][j]["line_color"] = color

    def paint_rect(self, row, col, rows=1, cols=1, color="black", cover=False):
        self.t.pencolor(self.__mat[row][col]["line_color"])
        self.t.fillcolor(color)
        self.t.pu()
        self.t.setheading(0)
        self.t.goto(self.__left + col * self.__grid_size, self.__top - row * self.__grid_size) 
        self.t.pd()
        self.t.begin_fill()
        for i in range(2):
            self.t.fd(self.__grid_size * cols)
            self.t.rt(90)
            self.t.fd(self.__grid_size * rows)
            self.t.rt(90)
        self.t.end_fill()
        if cover:
            for i in range(row, row + rows):
                for j in range(col, col + cols):
                    self.__mat[i][j] = [
                        {"text":"", "line_color":"black", "text_color":"black", "font":("Times", 24)}
                    ]
        else:
            for i in range(row, row + rows):
                for j in range(col, col + cols):
                    self.draw_text(row, col,
                                   self.__mat[i][j]["text"], self.__mat[i][j]["text_color"],
                                   self.__mat[i][j]["font"])

    def draw_circle(self, row, col, color="black"):
        self.t.pencolor(color)
        self.t.pu()
        self.t.goto(self.__left + col * self.__grid_size, self.__top - row * self.__grid_size) 
        self.t.pd()
        self.t.circle(self.__grid_size / 2)

    
    def draw_text(self, row, col, text, color="black", font=("Times", 24)):
        self.__mat[row][col]["text"] = text
        self.__mat[row][col]["text_color"] = color
        self.__mat[row][col]["font"] = font
        self.t.pencolor(color)
        self.t.pu()
        text_width = len(text) * font[1] / 2
        text_height = font[1] * 1.16 

        self.t.goto(self.__left + col * self.__grid_size + (self.__grid_size - text_width) / 2,
               self.__top - row * self.__grid_size - (self.__grid_size + text_height) / 2) 
        self.t.pd()
        self.t.write(text, font=font)
        self.t.pu()

    def draw_line(self, from_row, from_col, to_row, to_col, color="black"):
        self.t.pencolor(color)
        self.t.pu()
        self.t.goto(self.__left + from_col * self.__grid_size,
                    self.__top - from_row * self.__grid_size)
        self.t.pd()
        self.t.goto(self.__left + to_col * self.__grid_size,
                    self.__top - to_row * self.__grid_size)

    def save2eps(self, file):
        turtle.getcanvas().postscript(file = file)

if __name__ == '__main__':

    rabit = Rabit(11)
    rabit.begin_hide()
    for i in range(11):
        for j in range(11):
            rabit.draw_rect(i, j, color="red")
    rabit.end_hide()

    rabit.draw_circle(1.5, 1.5)
    for i in range(11):
        rabit.draw_text(i, 1, "1")
        rabit.paint_rect(i, 1, color="red")
        rabit.paint_rect(i, 1, color="blue", cover=True)

    turtle.done()




import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(name='rabit',
      version='0.2',
      description='This project implements a simple API of turtle to draw line, rectangle, circle, text and convert the process into gif.',
      long_description=long_description,
      long_description_content_type='text/markdown',
      url='http://github.com/ackeraa/rabit',
      author='Acker',
      author_email='acker2880@gmail.com',
      license='MIT',
      packages=setuptools.find_packages(),
      install_requires=[
          'pythonturtle',
          'imageio'
      ],
      include_package_data = True,
      platforms = 'any',
      python_requires='>=3.6'
 )
